/*============================================================================
  File:     Indexing for Joins - Credit.sql

  Summary:  Various tests and index access patterns using 5 table joins.
  
  Date:     June 2005

  SQL Server Version: 9.00.1116.08 (April CTP)
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

SET STATISTICS IO ON
-- Turn Graphical Showplan ON (Ctrl+K)

USE Credit
go

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

-- Where is all the cost? (Highest number in showplan output...)
	-- Should be in charge!

-- SO - what does the charge table need in this query?
-- Search List		charge_amt
-- Join Cond1		member_no
-- Join Cond2		provider_no
-- Join Cond3		statement_no
-- Search Arg		charge_amt

-- Create "test" Indexes with priority (first column - or high order element) 
-- given to the search argument or the join and come up with some test cases... 
-- Try out the query again and see which one it chooses! Always put the cols of the
-- select list last!!!! They are the least significant!

-- First try just covering the sarg and join - in this case there is no sarg
CREATE INDEX testa_1 ON charge(charge_amt, member_no)
CREATE INDEX testa_2 ON charge(member_no, charge_amt)
CREATE INDEX testa_3 ON charge(provider_no, charge_amt)
CREATE INDEX testa_4 ON charge(charge_amt, provider_no)

-- Compare against first plan
SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (1))
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

-- What about covering the QUERY...
CREATE INDEX testb_1 ON charge(charge_amt, member_no, provider_no, statement_no)
CREATE INDEX testb_2 ON charge(member_no, charge_amt, provider_no, statement_no)
CREATE INDEX testb_3 ON charge(provider_no, charge_amt, member_no, statement_no)
CREATE INDEX testb_4 ON charge(charge_amt, provider_no, member_no, statement_no)

-- Run the query again to see which one it chooses?

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (1))
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

-- to drop all of the indexes:
DROP INDEX charge.testa_1
DROP INDEX charge.testa_2
DROP INDEX charge.testa_3
DROP INDEX charge.testa_4
DROP INDEX charge.testb_1
DROP INDEX charge.testb_2
DROP INDEX charge.testb_3
DROP INDEX charge.testb_4

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c WITH (INDEX (1))
		INNER JOIN dbo.provider AS p WITH (INDEX(provider_ident))
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m WITH (INDEX (member_ident)) 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s WITH (INDEX (statement_member_link))
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r WITH (INDEX (region_ident))
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c WITH (INDEX ([_dta_index_charge_6_229575856__K2_K6_K3_K7]))
		INNER JOIN dbo.provider AS p WITH (INDEX(provider_ident))
			ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m WITH (INDEX (member_ident)) 
			ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s WITH (INDEX (statement_member_link))
			ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r WITH (INDEX (region_ident))
			ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500

SELECT s.statement_no, c.charge_amt, p.provider_name, m.lastname
	FROM dbo.Charge AS c
		INNER JOIN dbo.provider AS p ON p.provider_no = c.provider_no
		INNER JOIN dbo.member AS m ON c.member_no = m.member_no
		INNER JOIN dbo.statement AS s ON c.statement_no = s.statement_no
		INNER JOIN dbo.region AS r ON r.region_no = m.region_no
WHERE r.region_name = 'Japan'
	AND c.charge_amt > 2500
